# Generated by the protocol buffer compiler.  DO NOT EDIT!

from google.protobuf import descriptor
from google.protobuf import message
from google.protobuf import reflection
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)


import locker_pb2

DESCRIPTOR = descriptor.FileDescriptor(
  name='upload.proto',
  package='',
  serialized_pb='\n\x0cupload.proto\x1a\x0clocker.proto\"\x99\x01\n\x0eResponseStatus\x12\x33\n\rresponse_code\x18\x01 \x02(\x0e\x32\x1c.ResponseStatus.ResponseCode\"R\n\x0cResponseCode\x12\x06\n\x02OK\x10\x01\x12\x12\n\x0e\x41LREADY_EXISTS\x10\x02\x12\x0e\n\nSOFT_ERROR\x10\x03\x12\x16\n\x12METADATA_TOO_LARGE\x10\x04\"\x8f\x01\n\x0fUploadOperation\x12-\n\toperation\x18\x01 \x02(\x0e\x32\x1a.UploadOperation.Operation\"M\n\tOperation\x12\x14\n\x10OPERATION_CREATE\x10\x01\x12\x14\n\x10OPERATION_MODIFY\x10\x02\x12\x14\n\x10OPERATION_DELETE\x10\x03\"\xa5\x01\n\rChallengeInfo\x12\x17\n\x0f\x63lient_track_id\x18\x01 \x02(\t\x12\x14\n\x0cstart_millis\x18\x02 \x02(\x05\x12\x17\n\x0f\x64uration_millis\x18\x03 \x02(\x05\x12\x19\n\x11\x63hallenge_user_id\x18\x04 \x01(\t\x12\x1b\n\x13\x63hallenge_timestamp\x18\x05 \x01(\x03\x12\x14\n\x0c\x65xpect_match\x18\x06 \x01(\x08\"P\n\x13SignedChallengeInfo\x12&\n\x0e\x63hallenge_info\x18\x01 \x02(\x0b\x32\x0e.ChallengeInfo\x12\x11\n\tsignature\x18\x02 \x02(\x0c\"\xb9\x01\n\x0bTrackSample\x12\x15\n\x05track\x18\x01 \x02(\x0b\x32\x06.Track\x12\x33\n\x15signed_challenge_info\x18\x02 \x02(\x0b\x32\x14.SignedChallengeInfo\x12\x0e\n\x06sample\x18\x03 \x02(\x0c\x12)\n\rsample_format\x18\x04 \x01(\x0e\x32\x12.Track.ContentType\x12#\n\x0euser_album_art\x18\x05 \x01(\x0b\x32\x0b.ImageUnion\"u\n\x15UploadPlaylistRequest\x12*\n\x10upload_operation\x18\x01 \x02(\x0b\x32\x10.UploadOperation\x12\x1b\n\x08playlist\x18\x02 \x03(\x0b\x32\t.Playlist\x12\x13\n\x0buploader_id\x18\x03 \x02(\t\"b\n\x10PlaylistResponse\x12(\n\x0fresponse_status\x18\x01 \x02(\x0b\x32\x0f.ResponseStatus\x12\x11\n\tclient_id\x18\x02 \x01(\t\x12\x11\n\tserver_id\x18\x03 \x01(\t\"F\n\x16UploadPlaylistResponse\x12,\n\x11playlist_response\x18\x01 \x03(\x0b\x32\x11.PlaylistResponse\"\x85\x01\n\x1aUploadPlaylistEntryRequest\x12*\n\x10upload_operation\x18\x01 \x02(\x0b\x32\x10.UploadOperation\x12&\n\x0eplaylist_entry\x18\x02 \x03(\x0b\x32\x0e.PlaylistEntry\x12\x13\n\x0buploader_id\x18\x03 \x02(\t\"g\n\x15PlaylistEntryResponse\x12(\n\x0fresponse_status\x18\x01 \x02(\x0b\x32\x0f.ResponseStatus\x12\x11\n\tclient_id\x18\x02 \x01(\t\x12\x11\n\tserver_id\x18\x03 \x01(\t\"V\n\x1bUploadPlaylistEntryResponse\x12\x37\n\x17playlist_entry_response\x18\x01 \x03(\x0b\x32\x16.PlaylistEntryResponse\"C\n\x15UploadMetadataRequest\x12\x15\n\x05track\x18\x01 \x03(\x0b\x32\x06.Track\x12\x13\n\x0buploader_id\x18\x02 \x02(\t\"\x98\x01\n\x18UpdateUploadStateRequest\x12\x34\n\x05state\x18\x01 \x02(\x0e\x32%.UpdateUploadStateRequest.UploadState\x12\x13\n\x0buploader_id\x18\x02 \x02(\t\"1\n\x0bUploadState\x12\t\n\x05START\x10\x01\x12\n\n\x06PAUSED\x10\x02\x12\x0b\n\x07STOPPED\x10\x03\"M\n\x12\x43lientStateRequest\x12\x13\n\x0buploader_id\x18\x01 \x02(\t\x12\"\n\x1aget_purchased_tracks_since\x18\x02 \x01(\x03\"\xaf\x01\n\x13\x43lientStateResponse\x12\x1a\n\x12locker_track_limit\x18\x01 \x01(\x03\x12\x1c\n\x14user_songs_in_locker\x18\x02 \x01(\x03\x12\x1e\n\x16track_size_limit_in_mb\x18\x03 \x01(\x05\x12#\n\x1buser_purchased_tracks_since\x18\x04 \x01(\x03\x12\x19\n\x11total_track_count\x18\x05 \x01(\x03\"\x95\x01\n\x16UploadMetadataResponse\x12\x33\n\x15signed_challenge_info\x18\x01 \x03(\x0b\x32\x14.SignedChallengeInfo\x12\x11\n\tupload_id\x18\x02 \x03(\t\x12\x33\n\x15track_sample_response\x18\x03 \x03(\x0b\x32\x14.TrackSampleResponse\"\xa5\x03\n\x13TrackSampleResponse\x12\x17\n\x0f\x63lient_track_id\x18\x01 \x02(\t\x12\x38\n\rresponse_code\x18\x02 \x02(\x0e\x32!.TrackSampleResponse.ResponseCode\x12\x17\n\x0fserver_track_id\x18\x03 \x01(\t\x12\x15\n\ralbum_art_url\x18\x04 \x01(\t\x12\x14\n\x0c\x65xpect_match\x18\x05 \x01(\x08\"\xf4\x01\n\x0cResponseCode\x12\x0b\n\x07MATCHED\x10\x01\x12\x14\n\x10UPLOAD_REQUESTED\x10\x02\x12\x15\n\x11INVALID_SIGNATURE\x10\x03\x12\x12\n\x0e\x41LREADY_EXISTS\x10\x04\x12\x13\n\x0fTRANSIENT_ERROR\x10\x05\x12\x13\n\x0fPERMANENT_ERROR\x10\x06\x12\x1d\n\x19TRACK_COUNT_LIMIT_REACHED\x10\x07\x12\x16\n\x12REJECT_STORE_TRACK\x10\x08\x12\x1f\n\x1bREJECT_STORE_TRACK_BY_LABEL\x10\t\x12\x14\n\x10REJECT_DRM_TRACK\x10\n\"N\n\x13UploadSampleRequest\x12\"\n\x0ctrack_sample\x18\x01 \x03(\x0b\x32\x0c.TrackSample\x12\x13\n\x0buploader_id\x18\x02 \x02(\t\"K\n\x14UploadSampleResponse\x12\x33\n\x15track_sample_response\x18\x01 \x03(\x0b\x32\x14.TrackSampleResponse\";\n\nImageUnion\x12\x16\n\x0euser_album_art\x18\x01 \x01(\x0c\x12\x15\n\ralbum_art_url\x18\x02 \x01(\t\"\xaf\x07\n\x0eUploadResponse\x12\x33\n\rresponse_type\x18\x01 \x01(\x0e\x32\x1c.UploadResponse.ResponseType\x12\x32\n\x11metadata_response\x18\x02 \x01(\x0b\x32\x17.UploadMetadataResponse\x12\x32\n\x11playlist_response\x18\x03 \x01(\x0b\x32\x17.UploadPlaylistResponse\x12=\n\x17playlist_entry_response\x18\x04 \x01(\x0b\x32\x1c.UploadPlaylistEntryResponse\x12.\n\x0fsample_response\x18\x05 \x01(\x0b\x32\x15.UploadSampleResponse\x12*\n\x10getjobs_response\x18\x07 \x01(\x0b\x32\x10.GetJobsResponse\x12\x32\n\x14\x63lientstate_response\x18\x08 \x01(\x0b\x32\x14.ClientStateResponse\x12\x1d\n\x06policy\x18\x06 \x01(\x0b\x32\r.ClientPolicy\x12/\n\x0b\x61uth_status\x18\x0b \x01(\x0e\x32\x1a.UploadResponse.AuthStatus\x12\x12\n\nauth_error\x18\x0c \x01(\x08\"\xfa\x01\n\x0cResponseType\x12\x15\n\x11METADATA_RESPONSE\x10\x01\x12\x15\n\x11PLAYLIST_RESPONSE\x10\x02\x12\x1b\n\x17PLAYLIST_ENTRY_RESPONSE\x10\x03\x12\x13\n\x0fSAMPLE_RESPONSE\x10\x04\x12\x14\n\x10GETJOBS_RESPONSE\x10\x05\x12\x11\n\rAUTH_RESPONSE\x10\x06\x12\x19\n\x15\x43LIENT_STATE_RESPONSE\x10\x07\x12 \n\x1cUPDATE_UPLOAD_STATE_RESPONSE\x10\x08\x12$\n DELETE_UPLOAD_REQUESTED_RESPONSE\x10\t\"\xcf\x01\n\nAuthStatus\x12\x06\n\x02OK\x10\x08\x12\x15\n\x11MAX_LIMIT_REACHED\x10\t\x12!\n\x1d\x43LIENT_BOUND_TO_OTHER_ACCOUNT\x10\n\x12\x19\n\x15\x43LIENT_NOT_AUTHORIZED\x10\x0b\x12\"\n\x1eMAX_PER_MACHINE_USERS_EXCEEDED\x10\x0c\x12\x17\n\x13\x43LIENT_PLEASE_RETRY\x10\r\x12\x12\n\x0eNOT_SUBSCRIBED\x10\x0e\x12\x13\n\x0fINVALID_REQUEST\x10\x0f\"\x9c\x01\n\x0eTracksToUpload\x12\x11\n\tclient_id\x18\x01 \x02(\t\x12\x11\n\tserver_id\x18\x02 \x02(\t\x12+\n\x06status\x18\x05 \x02(\x0e\x32\x1b.TracksToUpload.TrackStatus\"7\n\x0bTrackStatus\x12\x12\n\x0e\x46ORCE_REUPLOAD\x10\x03\x12\x14\n\x10UPLOAD_REQUESTED\x10\x04\"%\n\x0eGetJobsRequest\x12\x13\n\x0buploader_id\x18\x01 \x02(\t\"X\n\x0fGetJobsResponse\x12)\n\x10tracks_to_upload\x18\x01 \x03(\x0b\x32\x0f.TracksToUpload\x12\x1a\n\x12get_tracks_success\x18\x02 \x02(\x08\";\n\rUpAuthRequest\x12\x13\n\x0buploader_id\x18\x01 \x02(\t\x12\x15\n\rfriendly_name\x18\x02 \x01(\t\"3\n\x1c\x44\x65leteUploadRequestedRequest\x12\x13\n\x0buploader_id\x18\x01 \x02(\t\"\x83\x02\n\x0c\x43lientPolicy\x12\x15\n\rpause_uploads\x18\x01 \x01(\x08\x12\r\n\x05\x61\x62ort\x18\x02 \x01(\x08\x12\x18\n\x10retry_in_minutes\x18\x03 \x01(\x05\x12\x1a\n\x12\x62\x61ndwidth_cap_kbps\x18\x04 \x01(\x05\x12\x17\n\x0fpause_downloads\x18\x05 \x01(\x08\x12#\n\x1b\x64ownload_bandwidth_cap_kbps\x18\x06 \x01(\x05\x12\x16\n\x0esend_analytics\x18\x07 \x01(\x08\x12\x15\n\ranalytics_url\x18\x08 \x01(\t\x12\x16\n\x0e\x65nable_gapless\x18\t \x01(\x08\x12\x12\n\nenable_m4p\x18\n \x01(\x08\"X\n\x1dUploadMetadataInternalRequest\x12\x0e\n\x06gaiaid\x18\x01 \x02(\x04\x12\'\n\x07request\x18\x02 \x02(\x0b\x32\x16.UploadMetadataRequest\"\xc3\x02\n\x13OverrideConfigValue\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\x12\x37\n\x08priority\x18\x03 \x01(\x0e\x32%.OverrideConfigValue.OverridePriority\"\xd6\x01\n\x10OverridePriority\x12\x1d\n\x10\x44\x45\x46\x41ULT_PRIORITY\x10\xff\xff\xff\xff\xff\xff\xff\xff\xff\x01\x12\x13\n\x0fLOWEST_PRIORITY\x10\x64\x12\x11\n\x0cLOW_PRIORITY\x10\xc8\x01\x12\x14\n\x0fMEDIUM_PRIORITY\x10\xac\x02\x12\x12\n\rHIGH_PRIORITY\x10\x90\x03\x12\x15\n\x10HIGHEST_PRIORITY\x10\xf4\x03\x12\x1c\n\x17\x44\x45\x46\x41ULT_SERVER_PRIORITY\x10\xac\x02\x12\x1c\n\x17\x44\x45\x46\x41ULT_CLIENT_PRIORITY\x10\x90\x03\"D\n\x1dOverrideConfigValueCollection\x12#\n\x05value\x18\x01 \x03(\x0b\x32\x14.OverrideConfigValue')



_RESPONSESTATUS_RESPONSECODE = descriptor.EnumDescriptor(
  name='ResponseCode',
  full_name='ResponseStatus.ResponseCode',
  filename=None,
  file=DESCRIPTOR,
  values=[
    descriptor.EnumValueDescriptor(
      name='OK', index=0, number=1,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='ALREADY_EXISTS', index=1, number=2,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='SOFT_ERROR', index=2, number=3,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='METADATA_TOO_LARGE', index=3, number=4,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=102,
  serialized_end=184,
)

_UPLOADOPERATION_OPERATION = descriptor.EnumDescriptor(
  name='Operation',
  full_name='UploadOperation.Operation',
  filename=None,
  file=DESCRIPTOR,
  values=[
    descriptor.EnumValueDescriptor(
      name='OPERATION_CREATE', index=0, number=1,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='OPERATION_MODIFY', index=1, number=2,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='OPERATION_DELETE', index=2, number=3,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=253,
  serialized_end=330,
)

_UPDATEUPLOADSTATEREQUEST_UPLOADSTATE = descriptor.EnumDescriptor(
  name='UploadState',
  full_name='UpdateUploadStateRequest.UploadState',
  filename=None,
  file=DESCRIPTOR,
  values=[
    descriptor.EnumValueDescriptor(
      name='START', index=0, number=1,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='PAUSED', index=1, number=2,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='STOPPED', index=2, number=3,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1563,
  serialized_end=1612,
)

_TRACKSAMPLERESPONSE_RESPONSECODE = descriptor.EnumDescriptor(
  name='ResponseCode',
  full_name='TrackSampleResponse.ResponseCode',
  filename=None,
  file=DESCRIPTOR,
  values=[
    descriptor.EnumValueDescriptor(
      name='MATCHED', index=0, number=1,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='UPLOAD_REQUESTED', index=1, number=2,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='INVALID_SIGNATURE', index=2, number=3,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='ALREADY_EXISTS', index=3, number=4,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='TRANSIENT_ERROR', index=4, number=5,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='PERMANENT_ERROR', index=5, number=6,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='TRACK_COUNT_LIMIT_REACHED', index=6, number=7,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='REJECT_STORE_TRACK', index=7, number=8,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='REJECT_STORE_TRACK_BY_LABEL', index=8, number=9,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='REJECT_DRM_TRACK', index=9, number=10,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=2201,
  serialized_end=2445,
)

_UPLOADRESPONSE_RESPONSETYPE = descriptor.EnumDescriptor(
  name='ResponseType',
  full_name='UploadResponse.ResponseType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    descriptor.EnumValueDescriptor(
      name='METADATA_RESPONSE', index=0, number=1,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='PLAYLIST_RESPONSE', index=1, number=2,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='PLAYLIST_ENTRY_RESPONSE', index=2, number=3,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='SAMPLE_RESPONSE', index=3, number=4,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='GETJOBS_RESPONSE', index=4, number=5,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='AUTH_RESPONSE', index=5, number=6,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='CLIENT_STATE_RESPONSE', index=6, number=7,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='UPDATE_UPLOAD_STATE_RESPONSE', index=7, number=8,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='DELETE_UPLOAD_REQUESTED_RESPONSE', index=8, number=9,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=3149,
  serialized_end=3399,
)

_UPLOADRESPONSE_AUTHSTATUS = descriptor.EnumDescriptor(
  name='AuthStatus',
  full_name='UploadResponse.AuthStatus',
  filename=None,
  file=DESCRIPTOR,
  values=[
    descriptor.EnumValueDescriptor(
      name='OK', index=0, number=8,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='MAX_LIMIT_REACHED', index=1, number=9,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='CLIENT_BOUND_TO_OTHER_ACCOUNT', index=2, number=10,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='CLIENT_NOT_AUTHORIZED', index=3, number=11,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='MAX_PER_MACHINE_USERS_EXCEEDED', index=4, number=12,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='CLIENT_PLEASE_RETRY', index=5, number=13,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='NOT_SUBSCRIBED', index=6, number=14,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='INVALID_REQUEST', index=7, number=15,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=3402,
  serialized_end=3609,
)

_TRACKSTOUPLOAD_TRACKSTATUS = descriptor.EnumDescriptor(
  name='TrackStatus',
  full_name='TracksToUpload.TrackStatus',
  filename=None,
  file=DESCRIPTOR,
  values=[
    descriptor.EnumValueDescriptor(
      name='FORCE_REUPLOAD', index=0, number=3,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='UPLOAD_REQUESTED', index=1, number=4,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=3713,
  serialized_end=3768,
)

_OVERRIDECONFIGVALUE_OVERRIDEPRIORITY = descriptor.EnumDescriptor(
  name='OverridePriority',
  full_name='OverrideConfigValue.OverridePriority',
  filename=None,
  file=DESCRIPTOR,
  values=[
    descriptor.EnumValueDescriptor(
      name='DEFAULT_PRIORITY', index=0, number=-1,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='LOWEST_PRIORITY', index=1, number=100,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='LOW_PRIORITY', index=2, number=200,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='MEDIUM_PRIORITY', index=3, number=300,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='HIGH_PRIORITY', index=4, number=400,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='HIGHEST_PRIORITY', index=5, number=500,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='DEFAULT_SERVER_PRIORITY', index=6, number=300,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='DEFAULT_CLIENT_PRIORITY', index=7, number=400,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=4475,
  serialized_end=4689,
)


_RESPONSESTATUS = descriptor.Descriptor(
  name='ResponseStatus',
  full_name='ResponseStatus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='response_code', full_name='ResponseStatus.response_code', index=0,
      number=1, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _RESPONSESTATUS_RESPONSECODE,
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=31,
  serialized_end=184,
)


_UPLOADOPERATION = descriptor.Descriptor(
  name='UploadOperation',
  full_name='UploadOperation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='operation', full_name='UploadOperation.operation', index=0,
      number=1, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _UPLOADOPERATION_OPERATION,
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=187,
  serialized_end=330,
)


_CHALLENGEINFO = descriptor.Descriptor(
  name='ChallengeInfo',
  full_name='ChallengeInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='client_track_id', full_name='ChallengeInfo.client_track_id', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='start_millis', full_name='ChallengeInfo.start_millis', index=1,
      number=2, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='duration_millis', full_name='ChallengeInfo.duration_millis', index=2,
      number=3, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='challenge_user_id', full_name='ChallengeInfo.challenge_user_id', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='challenge_timestamp', full_name='ChallengeInfo.challenge_timestamp', index=4,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='expect_match', full_name='ChallengeInfo.expect_match', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=333,
  serialized_end=498,
)


_SIGNEDCHALLENGEINFO = descriptor.Descriptor(
  name='SignedChallengeInfo',
  full_name='SignedChallengeInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='challenge_info', full_name='SignedChallengeInfo.challenge_info', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='signature', full_name='SignedChallengeInfo.signature', index=1,
      number=2, type=12, cpp_type=9, label=2,
      has_default_value=False, default_value="",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=500,
  serialized_end=580,
)


_TRACKSAMPLE = descriptor.Descriptor(
  name='TrackSample',
  full_name='TrackSample',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='track', full_name='TrackSample.track', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='signed_challenge_info', full_name='TrackSample.signed_challenge_info', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='sample', full_name='TrackSample.sample', index=2,
      number=3, type=12, cpp_type=9, label=2,
      has_default_value=False, default_value="",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='sample_format', full_name='TrackSample.sample_format', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='user_album_art', full_name='TrackSample.user_album_art', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=583,
  serialized_end=768,
)


_UPLOADPLAYLISTREQUEST = descriptor.Descriptor(
  name='UploadPlaylistRequest',
  full_name='UploadPlaylistRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='upload_operation', full_name='UploadPlaylistRequest.upload_operation', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='playlist', full_name='UploadPlaylistRequest.playlist', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='uploader_id', full_name='UploadPlaylistRequest.uploader_id', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=770,
  serialized_end=887,
)


_PLAYLISTRESPONSE = descriptor.Descriptor(
  name='PlaylistResponse',
  full_name='PlaylistResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='response_status', full_name='PlaylistResponse.response_status', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='client_id', full_name='PlaylistResponse.client_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='server_id', full_name='PlaylistResponse.server_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=889,
  serialized_end=987,
)


_UPLOADPLAYLISTRESPONSE = descriptor.Descriptor(
  name='UploadPlaylistResponse',
  full_name='UploadPlaylistResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='playlist_response', full_name='UploadPlaylistResponse.playlist_response', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=989,
  serialized_end=1059,
)


_UPLOADPLAYLISTENTRYREQUEST = descriptor.Descriptor(
  name='UploadPlaylistEntryRequest',
  full_name='UploadPlaylistEntryRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='upload_operation', full_name='UploadPlaylistEntryRequest.upload_operation', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='playlist_entry', full_name='UploadPlaylistEntryRequest.playlist_entry', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='uploader_id', full_name='UploadPlaylistEntryRequest.uploader_id', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1062,
  serialized_end=1195,
)


_PLAYLISTENTRYRESPONSE = descriptor.Descriptor(
  name='PlaylistEntryResponse',
  full_name='PlaylistEntryResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='response_status', full_name='PlaylistEntryResponse.response_status', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='client_id', full_name='PlaylistEntryResponse.client_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='server_id', full_name='PlaylistEntryResponse.server_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1197,
  serialized_end=1300,
)


_UPLOADPLAYLISTENTRYRESPONSE = descriptor.Descriptor(
  name='UploadPlaylistEntryResponse',
  full_name='UploadPlaylistEntryResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='playlist_entry_response', full_name='UploadPlaylistEntryResponse.playlist_entry_response', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1302,
  serialized_end=1388,
)


_UPLOADMETADATAREQUEST = descriptor.Descriptor(
  name='UploadMetadataRequest',
  full_name='UploadMetadataRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='track', full_name='UploadMetadataRequest.track', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='uploader_id', full_name='UploadMetadataRequest.uploader_id', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1390,
  serialized_end=1457,
)


_UPDATEUPLOADSTATEREQUEST = descriptor.Descriptor(
  name='UpdateUploadStateRequest',
  full_name='UpdateUploadStateRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='state', full_name='UpdateUploadStateRequest.state', index=0,
      number=1, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='uploader_id', full_name='UpdateUploadStateRequest.uploader_id', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _UPDATEUPLOADSTATEREQUEST_UPLOADSTATE,
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1460,
  serialized_end=1612,
)


_CLIENTSTATEREQUEST = descriptor.Descriptor(
  name='ClientStateRequest',
  full_name='ClientStateRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='uploader_id', full_name='ClientStateRequest.uploader_id', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='get_purchased_tracks_since', full_name='ClientStateRequest.get_purchased_tracks_since', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1614,
  serialized_end=1691,
)


_CLIENTSTATERESPONSE = descriptor.Descriptor(
  name='ClientStateResponse',
  full_name='ClientStateResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='locker_track_limit', full_name='ClientStateResponse.locker_track_limit', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='user_songs_in_locker', full_name='ClientStateResponse.user_songs_in_locker', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='track_size_limit_in_mb', full_name='ClientStateResponse.track_size_limit_in_mb', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='user_purchased_tracks_since', full_name='ClientStateResponse.user_purchased_tracks_since', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='total_track_count', full_name='ClientStateResponse.total_track_count', index=4,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1694,
  serialized_end=1869,
)


_UPLOADMETADATARESPONSE = descriptor.Descriptor(
  name='UploadMetadataResponse',
  full_name='UploadMetadataResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='signed_challenge_info', full_name='UploadMetadataResponse.signed_challenge_info', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='upload_id', full_name='UploadMetadataResponse.upload_id', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='track_sample_response', full_name='UploadMetadataResponse.track_sample_response', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1872,
  serialized_end=2021,
)


_TRACKSAMPLERESPONSE = descriptor.Descriptor(
  name='TrackSampleResponse',
  full_name='TrackSampleResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='client_track_id', full_name='TrackSampleResponse.client_track_id', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='response_code', full_name='TrackSampleResponse.response_code', index=1,
      number=2, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='server_track_id', full_name='TrackSampleResponse.server_track_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='album_art_url', full_name='TrackSampleResponse.album_art_url', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='expect_match', full_name='TrackSampleResponse.expect_match', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _TRACKSAMPLERESPONSE_RESPONSECODE,
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=2024,
  serialized_end=2445,
)


_UPLOADSAMPLEREQUEST = descriptor.Descriptor(
  name='UploadSampleRequest',
  full_name='UploadSampleRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='track_sample', full_name='UploadSampleRequest.track_sample', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='uploader_id', full_name='UploadSampleRequest.uploader_id', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=2447,
  serialized_end=2525,
)


_UPLOADSAMPLERESPONSE = descriptor.Descriptor(
  name='UploadSampleResponse',
  full_name='UploadSampleResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='track_sample_response', full_name='UploadSampleResponse.track_sample_response', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=2527,
  serialized_end=2602,
)


_IMAGEUNION = descriptor.Descriptor(
  name='ImageUnion',
  full_name='ImageUnion',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='user_album_art', full_name='ImageUnion.user_album_art', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value="",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='album_art_url', full_name='ImageUnion.album_art_url', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=2604,
  serialized_end=2663,
)


_UPLOADRESPONSE = descriptor.Descriptor(
  name='UploadResponse',
  full_name='UploadResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='response_type', full_name='UploadResponse.response_type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='metadata_response', full_name='UploadResponse.metadata_response', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='playlist_response', full_name='UploadResponse.playlist_response', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='playlist_entry_response', full_name='UploadResponse.playlist_entry_response', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='sample_response', full_name='UploadResponse.sample_response', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='getjobs_response', full_name='UploadResponse.getjobs_response', index=5,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='clientstate_response', full_name='UploadResponse.clientstate_response', index=6,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='policy', full_name='UploadResponse.policy', index=7,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='auth_status', full_name='UploadResponse.auth_status', index=8,
      number=11, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=8,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='auth_error', full_name='UploadResponse.auth_error', index=9,
      number=12, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _UPLOADRESPONSE_RESPONSETYPE,
    _UPLOADRESPONSE_AUTHSTATUS,
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=2666,
  serialized_end=3609,
)


_TRACKSTOUPLOAD = descriptor.Descriptor(
  name='TracksToUpload',
  full_name='TracksToUpload',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='client_id', full_name='TracksToUpload.client_id', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='server_id', full_name='TracksToUpload.server_id', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='status', full_name='TracksToUpload.status', index=2,
      number=5, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=3,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _TRACKSTOUPLOAD_TRACKSTATUS,
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=3612,
  serialized_end=3768,
)


_GETJOBSREQUEST = descriptor.Descriptor(
  name='GetJobsRequest',
  full_name='GetJobsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='uploader_id', full_name='GetJobsRequest.uploader_id', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=3770,
  serialized_end=3807,
)


_GETJOBSRESPONSE = descriptor.Descriptor(
  name='GetJobsResponse',
  full_name='GetJobsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='tracks_to_upload', full_name='GetJobsResponse.tracks_to_upload', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='get_tracks_success', full_name='GetJobsResponse.get_tracks_success', index=1,
      number=2, type=8, cpp_type=7, label=2,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=3809,
  serialized_end=3897,
)


_UPAUTHREQUEST = descriptor.Descriptor(
  name='UpAuthRequest',
  full_name='UpAuthRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='uploader_id', full_name='UpAuthRequest.uploader_id', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='friendly_name', full_name='UpAuthRequest.friendly_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=3899,
  serialized_end=3958,
)


_DELETEUPLOADREQUESTEDREQUEST = descriptor.Descriptor(
  name='DeleteUploadRequestedRequest',
  full_name='DeleteUploadRequestedRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='uploader_id', full_name='DeleteUploadRequestedRequest.uploader_id', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=3960,
  serialized_end=4011,
)


_CLIENTPOLICY = descriptor.Descriptor(
  name='ClientPolicy',
  full_name='ClientPolicy',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='pause_uploads', full_name='ClientPolicy.pause_uploads', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='abort', full_name='ClientPolicy.abort', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='retry_in_minutes', full_name='ClientPolicy.retry_in_minutes', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='bandwidth_cap_kbps', full_name='ClientPolicy.bandwidth_cap_kbps', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='pause_downloads', full_name='ClientPolicy.pause_downloads', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='download_bandwidth_cap_kbps', full_name='ClientPolicy.download_bandwidth_cap_kbps', index=5,
      number=6, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='send_analytics', full_name='ClientPolicy.send_analytics', index=6,
      number=7, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='analytics_url', full_name='ClientPolicy.analytics_url', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='enable_gapless', full_name='ClientPolicy.enable_gapless', index=8,
      number=9, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='enable_m4p', full_name='ClientPolicy.enable_m4p', index=9,
      number=10, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=4014,
  serialized_end=4273,
)


_UPLOADMETADATAINTERNALREQUEST = descriptor.Descriptor(
  name='UploadMetadataInternalRequest',
  full_name='UploadMetadataInternalRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='gaiaid', full_name='UploadMetadataInternalRequest.gaiaid', index=0,
      number=1, type=4, cpp_type=4, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='request', full_name='UploadMetadataInternalRequest.request', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=4275,
  serialized_end=4363,
)


_OVERRIDECONFIGVALUE = descriptor.Descriptor(
  name='OverrideConfigValue',
  full_name='OverrideConfigValue',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='key', full_name='OverrideConfigValue.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='value', full_name='OverrideConfigValue.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='priority', full_name='OverrideConfigValue.priority', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=-1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _OVERRIDECONFIGVALUE_OVERRIDEPRIORITY,
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=4366,
  serialized_end=4689,
)


_OVERRIDECONFIGVALUECOLLECTION = descriptor.Descriptor(
  name='OverrideConfigValueCollection',
  full_name='OverrideConfigValueCollection',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='value', full_name='OverrideConfigValueCollection.value', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=4691,
  serialized_end=4759,
)

_RESPONSESTATUS.fields_by_name['response_code'].enum_type = _RESPONSESTATUS_RESPONSECODE
_RESPONSESTATUS_RESPONSECODE.containing_type = _RESPONSESTATUS;
_UPLOADOPERATION.fields_by_name['operation'].enum_type = _UPLOADOPERATION_OPERATION
_UPLOADOPERATION_OPERATION.containing_type = _UPLOADOPERATION;
_SIGNEDCHALLENGEINFO.fields_by_name['challenge_info'].message_type = _CHALLENGEINFO
_TRACKSAMPLE.fields_by_name['track'].message_type = locker_pb2._TRACK
_TRACKSAMPLE.fields_by_name['signed_challenge_info'].message_type = _SIGNEDCHALLENGEINFO
_TRACKSAMPLE.fields_by_name['sample_format'].enum_type = locker_pb2._TRACK_CONTENTTYPE
_TRACKSAMPLE.fields_by_name['user_album_art'].message_type = _IMAGEUNION
_UPLOADPLAYLISTREQUEST.fields_by_name['upload_operation'].message_type = _UPLOADOPERATION
_UPLOADPLAYLISTREQUEST.fields_by_name['playlist'].message_type = locker_pb2._PLAYLIST
_PLAYLISTRESPONSE.fields_by_name['response_status'].message_type = _RESPONSESTATUS
_UPLOADPLAYLISTRESPONSE.fields_by_name['playlist_response'].message_type = _PLAYLISTRESPONSE
_UPLOADPLAYLISTENTRYREQUEST.fields_by_name['upload_operation'].message_type = _UPLOADOPERATION
_UPLOADPLAYLISTENTRYREQUEST.fields_by_name['playlist_entry'].message_type = locker_pb2._PLAYLISTENTRY
_PLAYLISTENTRYRESPONSE.fields_by_name['response_status'].message_type = _RESPONSESTATUS
_UPLOADPLAYLISTENTRYRESPONSE.fields_by_name['playlist_entry_response'].message_type = _PLAYLISTENTRYRESPONSE
_UPLOADMETADATAREQUEST.fields_by_name['track'].message_type = locker_pb2._TRACK
_UPDATEUPLOADSTATEREQUEST.fields_by_name['state'].enum_type = _UPDATEUPLOADSTATEREQUEST_UPLOADSTATE
_UPDATEUPLOADSTATEREQUEST_UPLOADSTATE.containing_type = _UPDATEUPLOADSTATEREQUEST;
_UPLOADMETADATARESPONSE.fields_by_name['signed_challenge_info'].message_type = _SIGNEDCHALLENGEINFO
_UPLOADMETADATARESPONSE.fields_by_name['track_sample_response'].message_type = _TRACKSAMPLERESPONSE
_TRACKSAMPLERESPONSE.fields_by_name['response_code'].enum_type = _TRACKSAMPLERESPONSE_RESPONSECODE
_TRACKSAMPLERESPONSE_RESPONSECODE.containing_type = _TRACKSAMPLERESPONSE;
_UPLOADSAMPLEREQUEST.fields_by_name['track_sample'].message_type = _TRACKSAMPLE
_UPLOADSAMPLERESPONSE.fields_by_name['track_sample_response'].message_type = _TRACKSAMPLERESPONSE
_UPLOADRESPONSE.fields_by_name['response_type'].enum_type = _UPLOADRESPONSE_RESPONSETYPE
_UPLOADRESPONSE.fields_by_name['metadata_response'].message_type = _UPLOADMETADATARESPONSE
_UPLOADRESPONSE.fields_by_name['playlist_response'].message_type = _UPLOADPLAYLISTRESPONSE
_UPLOADRESPONSE.fields_by_name['playlist_entry_response'].message_type = _UPLOADPLAYLISTENTRYRESPONSE
_UPLOADRESPONSE.fields_by_name['sample_response'].message_type = _UPLOADSAMPLERESPONSE
_UPLOADRESPONSE.fields_by_name['getjobs_response'].message_type = _GETJOBSRESPONSE
_UPLOADRESPONSE.fields_by_name['clientstate_response'].message_type = _CLIENTSTATERESPONSE
_UPLOADRESPONSE.fields_by_name['policy'].message_type = _CLIENTPOLICY
_UPLOADRESPONSE.fields_by_name['auth_status'].enum_type = _UPLOADRESPONSE_AUTHSTATUS
_UPLOADRESPONSE_RESPONSETYPE.containing_type = _UPLOADRESPONSE;
_UPLOADRESPONSE_AUTHSTATUS.containing_type = _UPLOADRESPONSE;
_TRACKSTOUPLOAD.fields_by_name['status'].enum_type = _TRACKSTOUPLOAD_TRACKSTATUS
_TRACKSTOUPLOAD_TRACKSTATUS.containing_type = _TRACKSTOUPLOAD;
_GETJOBSRESPONSE.fields_by_name['tracks_to_upload'].message_type = _TRACKSTOUPLOAD
_UPLOADMETADATAINTERNALREQUEST.fields_by_name['request'].message_type = _UPLOADMETADATAREQUEST
_OVERRIDECONFIGVALUE.fields_by_name['priority'].enum_type = _OVERRIDECONFIGVALUE_OVERRIDEPRIORITY
_OVERRIDECONFIGVALUE_OVERRIDEPRIORITY.containing_type = _OVERRIDECONFIGVALUE;
_OVERRIDECONFIGVALUECOLLECTION.fields_by_name['value'].message_type = _OVERRIDECONFIGVALUE
DESCRIPTOR.message_types_by_name['ResponseStatus'] = _RESPONSESTATUS
DESCRIPTOR.message_types_by_name['UploadOperation'] = _UPLOADOPERATION
DESCRIPTOR.message_types_by_name['ChallengeInfo'] = _CHALLENGEINFO
DESCRIPTOR.message_types_by_name['SignedChallengeInfo'] = _SIGNEDCHALLENGEINFO
DESCRIPTOR.message_types_by_name['TrackSample'] = _TRACKSAMPLE
DESCRIPTOR.message_types_by_name['UploadPlaylistRequest'] = _UPLOADPLAYLISTREQUEST
DESCRIPTOR.message_types_by_name['PlaylistResponse'] = _PLAYLISTRESPONSE
DESCRIPTOR.message_types_by_name['UploadPlaylistResponse'] = _UPLOADPLAYLISTRESPONSE
DESCRIPTOR.message_types_by_name['UploadPlaylistEntryRequest'] = _UPLOADPLAYLISTENTRYREQUEST
DESCRIPTOR.message_types_by_name['PlaylistEntryResponse'] = _PLAYLISTENTRYRESPONSE
DESCRIPTOR.message_types_by_name['UploadPlaylistEntryResponse'] = _UPLOADPLAYLISTENTRYRESPONSE
DESCRIPTOR.message_types_by_name['UploadMetadataRequest'] = _UPLOADMETADATAREQUEST
DESCRIPTOR.message_types_by_name['UpdateUploadStateRequest'] = _UPDATEUPLOADSTATEREQUEST
DESCRIPTOR.message_types_by_name['ClientStateRequest'] = _CLIENTSTATEREQUEST
DESCRIPTOR.message_types_by_name['ClientStateResponse'] = _CLIENTSTATERESPONSE
DESCRIPTOR.message_types_by_name['UploadMetadataResponse'] = _UPLOADMETADATARESPONSE
DESCRIPTOR.message_types_by_name['TrackSampleResponse'] = _TRACKSAMPLERESPONSE
DESCRIPTOR.message_types_by_name['UploadSampleRequest'] = _UPLOADSAMPLEREQUEST
DESCRIPTOR.message_types_by_name['UploadSampleResponse'] = _UPLOADSAMPLERESPONSE
DESCRIPTOR.message_types_by_name['ImageUnion'] = _IMAGEUNION
DESCRIPTOR.message_types_by_name['UploadResponse'] = _UPLOADRESPONSE
DESCRIPTOR.message_types_by_name['TracksToUpload'] = _TRACKSTOUPLOAD
DESCRIPTOR.message_types_by_name['GetJobsRequest'] = _GETJOBSREQUEST
DESCRIPTOR.message_types_by_name['GetJobsResponse'] = _GETJOBSRESPONSE
DESCRIPTOR.message_types_by_name['UpAuthRequest'] = _UPAUTHREQUEST
DESCRIPTOR.message_types_by_name['DeleteUploadRequestedRequest'] = _DELETEUPLOADREQUESTEDREQUEST
DESCRIPTOR.message_types_by_name['ClientPolicy'] = _CLIENTPOLICY
DESCRIPTOR.message_types_by_name['UploadMetadataInternalRequest'] = _UPLOADMETADATAINTERNALREQUEST
DESCRIPTOR.message_types_by_name['OverrideConfigValue'] = _OVERRIDECONFIGVALUE
DESCRIPTOR.message_types_by_name['OverrideConfigValueCollection'] = _OVERRIDECONFIGVALUECOLLECTION

class ResponseStatus(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _RESPONSESTATUS
  
  # @@protoc_insertion_point(class_scope:ResponseStatus)

class UploadOperation(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _UPLOADOPERATION
  
  # @@protoc_insertion_point(class_scope:UploadOperation)

class ChallengeInfo(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _CHALLENGEINFO
  
  # @@protoc_insertion_point(class_scope:ChallengeInfo)

class SignedChallengeInfo(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _SIGNEDCHALLENGEINFO
  
  # @@protoc_insertion_point(class_scope:SignedChallengeInfo)

class TrackSample(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _TRACKSAMPLE
  
  # @@protoc_insertion_point(class_scope:TrackSample)

class UploadPlaylistRequest(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _UPLOADPLAYLISTREQUEST
  
  # @@protoc_insertion_point(class_scope:UploadPlaylistRequest)

class PlaylistResponse(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _PLAYLISTRESPONSE
  
  # @@protoc_insertion_point(class_scope:PlaylistResponse)

class UploadPlaylistResponse(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _UPLOADPLAYLISTRESPONSE
  
  # @@protoc_insertion_point(class_scope:UploadPlaylistResponse)

class UploadPlaylistEntryRequest(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _UPLOADPLAYLISTENTRYREQUEST
  
  # @@protoc_insertion_point(class_scope:UploadPlaylistEntryRequest)

class PlaylistEntryResponse(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _PLAYLISTENTRYRESPONSE
  
  # @@protoc_insertion_point(class_scope:PlaylistEntryResponse)

class UploadPlaylistEntryResponse(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _UPLOADPLAYLISTENTRYRESPONSE
  
  # @@protoc_insertion_point(class_scope:UploadPlaylistEntryResponse)

class UploadMetadataRequest(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _UPLOADMETADATAREQUEST
  
  # @@protoc_insertion_point(class_scope:UploadMetadataRequest)

class UpdateUploadStateRequest(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _UPDATEUPLOADSTATEREQUEST
  
  # @@protoc_insertion_point(class_scope:UpdateUploadStateRequest)

class ClientStateRequest(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _CLIENTSTATEREQUEST
  
  # @@protoc_insertion_point(class_scope:ClientStateRequest)

class ClientStateResponse(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _CLIENTSTATERESPONSE
  
  # @@protoc_insertion_point(class_scope:ClientStateResponse)

class UploadMetadataResponse(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _UPLOADMETADATARESPONSE
  
  # @@protoc_insertion_point(class_scope:UploadMetadataResponse)

class TrackSampleResponse(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _TRACKSAMPLERESPONSE
  
  # @@protoc_insertion_point(class_scope:TrackSampleResponse)

class UploadSampleRequest(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _UPLOADSAMPLEREQUEST
  
  # @@protoc_insertion_point(class_scope:UploadSampleRequest)

class UploadSampleResponse(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _UPLOADSAMPLERESPONSE
  
  # @@protoc_insertion_point(class_scope:UploadSampleResponse)

class ImageUnion(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _IMAGEUNION
  
  # @@protoc_insertion_point(class_scope:ImageUnion)

class UploadResponse(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _UPLOADRESPONSE
  
  # @@protoc_insertion_point(class_scope:UploadResponse)

class TracksToUpload(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _TRACKSTOUPLOAD
  
  # @@protoc_insertion_point(class_scope:TracksToUpload)

class GetJobsRequest(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _GETJOBSREQUEST
  
  # @@protoc_insertion_point(class_scope:GetJobsRequest)

class GetJobsResponse(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _GETJOBSRESPONSE
  
  # @@protoc_insertion_point(class_scope:GetJobsResponse)

class UpAuthRequest(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _UPAUTHREQUEST
  
  # @@protoc_insertion_point(class_scope:UpAuthRequest)

class DeleteUploadRequestedRequest(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _DELETEUPLOADREQUESTEDREQUEST
  
  # @@protoc_insertion_point(class_scope:DeleteUploadRequestedRequest)

class ClientPolicy(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _CLIENTPOLICY
  
  # @@protoc_insertion_point(class_scope:ClientPolicy)

class UploadMetadataInternalRequest(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _UPLOADMETADATAINTERNALREQUEST
  
  # @@protoc_insertion_point(class_scope:UploadMetadataInternalRequest)

class OverrideConfigValue(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _OVERRIDECONFIGVALUE
  
  # @@protoc_insertion_point(class_scope:OverrideConfigValue)

class OverrideConfigValueCollection(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _OVERRIDECONFIGVALUECOLLECTION
  
  # @@protoc_insertion_point(class_scope:OverrideConfigValueCollection)

# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
The clients module exposes the main user-facing interfaces of gmusicapi.
"""

import copy
import logging
import os
from socket import gethostname
import time
import urllib
from uuid import getnode as getmac
import webbrowser

import httplib2  # included with oauth2client
from oauth2client.client import OAuth2WebServerFlow, TokenRevokeError
import oauth2client.file

import gmusicapi
from gmusicapi.gmtools import tools
from gmusicapi.exceptions import CallFailure, NotLoggedIn
from gmusicapi.protocol import webclient, musicmanager, upload_pb2, locker_pb2
from gmusicapi.utils import utils
import gmusicapi.session

OAUTH_FILEPATH = os.path.join(utils.my_appdirs.user_data_dir, 'oauth.cred')

# oauth client breaks if the dir doesn't exist
utils.make_sure_path_exists(os.path.dirname(OAUTH_FILEPATH), 0o700)


class _Base(object):
    """Factors out common client setup."""

    __metaclass__ = utils.DocstringInheritMeta

    num_clients = 0  # used to disambiguate loggers

    def __init__(self, logger_basename, debug_logging):
        """

        :param debug_logging: each Client has a ``logger`` member.
          The logger is named ``gmusicapi.<client class><client number>`` and
          will propogate to the ``gmusicapi`` root logger.

          If this param is ``True``, handlers will be configured to send
          this client's debug log output to disk,
          with warnings and above printed to stderr.
          `Appdirs <https://pypi.python.org/pypi/appdirs/1.2.0>`__
          ``user_log_dir`` is used by default. Users can run::

              from gmusicapi.utils import utils
              print utils.log_filepath

          to see the exact location on their system.

          If ``False``, no handlers will be configured;
          users must create their own handlers.

          Completely ignoring logging is dangerous and not recommended.
          The Google Music protocol can change at any time; if
          something were to go wrong, the logs would be necessary for
          recovery.
        """
        # this isn't correct if init is called more than once, so we log the
        # client name below to avoid confusion for people reading logs
        _Base.num_clients += 1

        logger_name = "gmusicapi.%s%s" % (logger_basename,
                                          _Base.num_clients)
        self.logger = logging.getLogger(logger_name)

        if debug_logging:
            utils.configure_debug_log_handlers(self.logger)

        self.logger.info("initialized")

    def _make_call(self, protocol, *args, **kwargs):
        """Returns the response of a protocol.Call.

        args/kwargs are passed to protocol.perform.

        CallFailure may be raised."""

        return protocol.perform(self.session, *args, **kwargs)

    def is_authenticated(self):
        """Returns ``True`` if the Api can make an authenticated request."""
        return self.session.is_authenticated

    def logout(self):
        """Forgets local authentication in this Api instance.
        Returns ``True`` on success."""

        self.session.logout()
        self.logger.info("logged out")
        return True


class Musicmanager(_Base):
    """Allows uploading by posing as Google's Music Manager.

    Musicmanager uses OAuth, so a plaintext email and password are not required
    when logging in.

    For most users, :func:`perform_oauth` should be run once per machine to
    store credentials to disk. Future calls to :func:`login` can use
    use the stored credentials by default.

    Alternatively, users can implement the OAuth flow themselves, then
    provide credentials directly to :func:`login`.
    """

    @staticmethod
    def perform_oauth(storage_filepath=OAUTH_FILEPATH, open_browser=False):
        """Provides a series of prompts for a user to follow to authenticate.
        Returns ``oauth2client.client.OAuth2Credentials`` when successful.

        In most cases, this should only be run once per machine to store
        credentials to disk, then never be needed again.

        If the user refuses to give access,
        ``oauth2client.client.FlowExchangeError`` is raised.

        :param storage_filepath: a filepath to write the credentials to,
          or ``None``
          to not write the credentials to disk (which is not recommended).

          `Appdirs <https://pypi.python.org/pypi/appdirs/1.2.0>`__
          ``user_data_dir`` is used by default. Users can run::

              import gmusicapi.clients
              print gmusicapi.clients.OAUTH_FILEPATH

          to see the exact location on their system.

        :param open_browser: if True, attempt to open the auth url
          in the system default web browser. The url will be printed
          regardless of this param's setting.
        """

        flow = OAuth2WebServerFlow(*musicmanager.oauth)

        auth_uri = flow.step1_get_authorize_url()
        print
        print "Visit the following url:\n %s" % auth_uri

        if open_browser:
            print
            print 'Opening your browser to it now...',
            webbrowser.open(auth_uri)
            print 'done.'
            print "If you don't see your browser, you can just copy and paste the url."
            print

        code = raw_input("Follow the prompts,"
                         " then paste the auth code here and hit enter: ")

        credentials = flow.step2_exchange(code)

        if storage_filepath is not None:
            storage = oauth2client.file.Storage(storage_filepath)
            storage.put(credentials)

        return credentials

    def __init__(self, debug_logging=True):
        self.session = gmusicapi.session.Musicmanager()

        super(Musicmanager, self).__init__(self.__class__.__name__, debug_logging)
        self.logout()

    def login(self, oauth_credentials=OAUTH_FILEPATH,
              uploader_id=None, uploader_name=None):
        """Authenticates the Music Manager using OAuth.
        Returns ``True`` on success, ``False`` on failure.

        Unlike the :class:`Webclient`, OAuth allows authentication without
        providing plaintext credentials to the application.

        In most cases, the default parameters should be acceptable. Users on
        virtual machines will want to provide `uploader_id`.

        :param oauth_credentials: ``oauth2client.client.OAuth2Credentials`` or the path to a
          ``oauth2client.file.Storage`` file. By default, the same default path used by
          :func:`perform_oauth` is used.

          Endusers will likely call :func:`perform_oauth` once to write
          credentials to disk and then ignore this parameter.

          This param
          is mostly intended to allow flexibility for developers of a
          3rd party service who intend to perform their own OAuth flow
          (eg on their website).

        :param uploader_id: a unique id as a MAC address, eg ``'00:11:22:33:AA:BB'``.
          This should only be provided in cases where the default
          (host MAC address incremented by 1) will not work.

          Upload behavior is undefined if a Music Manager uses the same id, especially when
          reporting bad matches.

          ``ValueError`` will be raised if this is provided but not in the proper form.

          ``OSError`` will be raised if this is not provided and a real MAC could not be
          determined (most common when running on a VPS).

          If provided, use the same id on all future runs for this machine,
          because of the upload device limit explained below.

        :param uploader_name: human-readable non-unique id; default is
          ``"<hostname> (gmusicapi-{version})"``.

          This doesn't appear to be a part of authentication at all.
          Registering with (id, name = X, Y) and logging in with
          (id, name = X, Z) works, and does not change the server-stored
          uploader_name.

        There are hard limits on how many upload devices can be registered; refer to `Google's
        docs <http://support.google.com/googleplay/bin/answer.py?hl=en&answer=1230356>`__. There
        have been limits on deauthorizing devices in the past, so it's smart not to register
        more devices than necessary.
        """

        return (self._oauth_login(oauth_credentials) and
                self._perform_upauth(uploader_id, uploader_name))

    def _oauth_login(self, oauth_credentials):
        """Auth ourselves to the MM oauth endpoint.

        Return True on success; see :py:func:`login` for params.
        """

        if isinstance(oauth_credentials, basestring):
            oauth_file = oauth_credentials
            storage = oauth2client.file.Storage(oauth_file)

            oauth_credentials = storage.get()
            if oauth_credentials is None:
                self.logger.warning("could not retrieve oauth credentials from '%s'", oauth_file)
                return False

        if not self.session.login(oauth_credentials):
            self.logger.warning("failed to authenticate")
            return False

        self.logger.info("oauth successful")

        return True

    def _perform_upauth(self, uploader_id, uploader_name):
        """Auth or register ourselves as an upload client.

        Return True on success; see :py:func:`login` for params.
        """

        if uploader_id is None:
            mac_int = getmac()
            if (mac_int >> 40) % 2:
                raise OSError('a valid MAC could not be determined.'
                              ' Provide uploader_id (and be'
                              ' sure to provide the same one on future runs).')

            else:
                #distinguish us from a Music Manager on this machine
                mac_int = (mac_int + 1) % (1 << 48)

            uploader_id = utils.create_mac_string(mac_int)

        if not utils.is_valid_mac(uploader_id):
            raise ValueError('uploader_id is not in a valid form.'
                             '\nProvide 6 pairs of hex digits'
                             ' with capital letters',
                             ' (eg "00:11:22:33:AA:BB")')

        if uploader_name is None:
            uploader_name = gethostname() + u" (gmusicapi-%s)" % gmusicapi.__version__

        try:
            # this is a MM-specific step that might register a new device.
            self._make_call(musicmanager.AuthenticateUploader,
                            uploader_id,
                            uploader_name)
            self.logger.info("successful upauth")
            self.uploader_id = uploader_id
            self.uploader_name = uploader_name

        except CallFailure:
            self.logger.exception("upauth failure")
            self.session.logout()
            return False

        return True

    def logout(self, revoke_oauth=False):
        """Forgets local authentication in this Client instance.

        :param revoke_oauth: if True, oauth credentials will be permanently
          revoked. If credentials came from a file, it will be deleted.

        Returns ``True`` on success."""

        # TODO the login/logout stuff is all over the place

        success = True

        if revoke_oauth:
            try:
                # this automatically deletes a Storage file, if present
                self.session._oauth_creds.revoke(httplib2.Http())
            except TokenRevokeError:
                self.logger.exception("could not revoke oauth credentials")
                success = False

        self.uploader_id = None
        self.uploader_name = None

        return success and super(Musicmanager, self).logout()

    # mostly copy-paste from Webclient.get_all_songs.
    # not worried about overlap in this case; the logic of either could change.
    def get_all_songs(self, incremental=False):
        """Returns a list of dictionaries, each with the following keys:
        ``('id', 'title', 'album', 'album_artist', 'artist', 'track_number',
        'track_size')``.

        :param incremental: if True, return a generator that yields lists
          of at most 1000 dictionaries
          as they are retrieved from the server. This can be useful for
          presenting a loading bar to a user.
        """

        to_return = self._get_all_songs()

        if not incremental:
            to_return = [song for chunk in to_return for song in chunk]

        return to_return

    @staticmethod
    def _track_info_to_dict(track_info):
        """Given a download_pb2.DownloadTrackInfo, return a dictionary."""
        # figure it's better to hardcode keys here than use introspection
        # and risk returning a new field all of a sudden.

        return dict((field, getattr(track_info, field)) for field in
                    ('id', 'title', 'album', 'album_artist', 'artist',
                     'track_number', 'track_size'))

    def _get_all_songs(self):
        """Return a generator of song chunks."""

        get_next_chunk = True

        # need to spoof .continuation_token access, and
        # can't add attrs to object(). Can with functions.

        lib_chunk = lambda: 0
        lib_chunk.continuation_token = None

        while get_next_chunk:
            lib_chunk = self._make_call(musicmanager.ListTracks,
                                        self.uploader_id,
                                        lib_chunk.continuation_token)

            yield [self._track_info_to_dict(info)
                   for info in lib_chunk.download_track_info]

            get_next_chunk = lib_chunk.HasField('continuation_token')

    def download_song(self, song_id):
        """Returns a tuple ``(u'suggested_filename', 'audio_bytestring')``.
        The filename
        will be what the Music Manager would save the file as,
        presented as a unicode string with the proper file extension.
        You don't have to use it if you don't want.


        :param song_id: a single song id.

        To write the song to disk, use something like::

            filename, audio = mm.download_song(an_id)

            # if open() throws a UnicodeEncodeError, either use
            #   filename.encode('utf-8')
            # or change your default encoding to something sane =)
            with open(filename, 'wb') as f:
                f.write(audio)

        Unlike with :py:func:`Webclient.get_song_download_info
        <gmusicapi.clients.Webclient.get_song_download_info>`,
        there is no download limit when using this interface.

        Also unlike the Webclient, downloading a track requires authentication.
        Returning a url does not suffice, since retrieving a track without auth
        will produce an http 500.
        """

        url = self._make_call(musicmanager.GetDownloadLink,
                              song_id,
                              self.uploader_id)['url']

        response = self._make_call(musicmanager.DownloadTrack, url)

        cd_header = response.headers['content-disposition']

        filename = urllib.unquote(cd_header.split("filename*=UTF-8''")[-1])
        filename = filename.decode('utf-8')

        return (filename, response.content)

    # def get_quota(self):
    #     """Returns a tuple of (allowed number of tracks, total tracks, available tracks)."""
    #     quota = self._mm_pb_call("client_state").quota
    #     #protocol incorrect here...
    #     return (quota.maximumTracks, quota.totalTracks, quota.availableTracks)

    @utils.accept_singleton(basestring)
    @utils.empty_arg_shortcircuit(return_code='{}')
    def upload(self, filepaths, transcode_quality=3, enable_matching=False):
        """Uploads the given filepaths.
        Any non-mp3 files will be `transcoded with avconv
        <https://github.com/simon-weber/Unofficial-Google-Music-API/
        blob/develop/gmusicapi/utils/utils.py#L18>`__ before being uploaded.

        Return a 3-tuple ``(uploaded, matched, not_uploaded)`` of dictionaries, eg::

            (
                {'<filepath>': '<new server id>'},               # uploaded
                {'<filepath>': '<new server id>'},               # matched
                {'<filepath>': '<reason, eg ALREADY_EXISTS>'}    # not uploaded
            )

        :param filepaths: a list of filepaths, or a single filepath.
        :param transcode_quality: if int, pass to avconv ``-qscale`` for libmp3lame
          (lower-better int, roughly corresponding to `hydrogenaudio -vX settings
          <http://wiki.hydrogenaudio.org/index.php?title=LAME#Recommended_encoder_settings>`__).
          If string, pass to avconv ``-ab`` (eg ``'128k'`` for an average bitrate of 128k). The
          default is ~175kbs vbr.

        :param enable_matching: if ``True``, attempt to use `scan and match
          <http://support.google.com/googleplay/bin/answer.py?hl=en&answer=2920799&topic=2450455>`__
          to avoid uploading every song.
          **WARNING**: currently, mismatched songs can *not* be fixed with the 'Fix Incorrect Match'
          button nor :py:func:`report_incorrect_match
          <gmusicapi.clients.Webclient.report_incorrect_match>`.
          They would have to be deleted and reuploaded with matching disabled
          (or with the Music Manager).
          Fixing matches from gmusicapi may be supported in a future release; see issue `#89
          <https://github.com/simon-weber/Unofficial-Google-Music-API/issues/89>`__.

        All Google-supported filetypes are supported; see `Google's documentation
        <http://support.google.com/googleplay/bin/answer.py?hl=en&answer=1100462>`__.

        Unlike Google's Music Manager, this function will currently allow the same song to
        be uploaded more than once if its tags are changed. This is subject to change in the future.

        If ``PERMANENT_ERROR`` is given as a not_uploaded reason, attempts to reupload will never
        succeed. The file will need to be changed before the server will reconsider it; the easiest
        way is to change metadata tags (it's not important that the tag be uploaded, just that the
        contents of the file change somehow).
        """

        if self.uploader_id is None or self.uploader_name is None:
            raise NotLoggedIn("Not authenticated as an upload device;"
                              " run Api.login(...perform_upload_auth=True...)"
                              " first.")

        #TODO there is way too much code in this function.

        #To return.
        uploaded = {}
        matched = {}
        not_uploaded = {}

        #Gather local information on the files.
        local_info = {}  # {clientid: (path, Track)}
        for path in filepaths:
            try:
                track = musicmanager.UploadMetadata.fill_track_info(path)
            except BaseException as e:
                self.logger.exception("problem gathering local info of '%r'", path)

                user_err_msg = str(e)

                if 'Non-ASCII strings must be converted to unicode' in str(e):
                    #This is a protobuf-specific error; they require either ascii or unicode.
                    #To keep behavior consistent, make no effort to guess - require users
                    # to decode first.
                    user_err_msg = ("nonascii bytestrings must be decoded to unicode"
                                    " (error: '%s')" % err_msg)

                not_uploaded[path] = user_err_msg
            else:
                local_info[track.client_id] = (path, track)

        if not local_info:
            return uploaded, matched, not_uploaded

        #TODO allow metadata faking

        #Upload metadata; the server tells us what to do next.
        res = self._make_call(musicmanager.UploadMetadata,
                              [track for (path, track) in local_info.values()],
                              self.uploader_id)

        #TODO checking for proper contents should be handled in verification
        md_res = res.metadata_response

        responses = [r for r in md_res.track_sample_response]
        sample_requests = [req for req in md_res.signed_challenge_info]

        #Send scan and match samples if requested.
        for sample_request in sample_requests:
            path, track = local_info[sample_request.challenge_info.client_track_id]

            bogus_sample = None
            if not enable_matching:
                bogus_sample = ''  # just send empty bytes

            try:
                res = self._make_call(musicmanager.ProvideSample,
                                      path, sample_request, track,
                                      self.uploader_id, bogus_sample)

            except (IOError, ValueError) as e:
                self.logger.warning("couldn't create scan and match sample for '%s': %s",
                                    path, str(e))
                not_uploaded[path] = str(e)
            else:
                responses.extend(res.sample_response.track_sample_response)

        #Read sample responses and prep upload requests.
        to_upload = {}  # {serverid: (path, Track, do_not_rematch?)}
        for sample_res in responses:
            path, track = local_info[sample_res.client_track_id]

            if sample_res.response_code == upload_pb2.TrackSampleResponse.MATCHED:
                self.logger.info("matched '%s' to sid %s", path, sample_res.server_track_id)

                if enable_matching:
                    matched[path] = sample_res.server_track_id
                else:
                    self.logger.exception("'%s' was matched without matching enabled", path)

            elif sample_res.response_code == upload_pb2.TrackSampleResponse.UPLOAD_REQUESTED:
                to_upload[sample_res.server_track_id] = (path, track, False)

            else:
                # there was a problem
                # report the symbolic name of the response code enum for debugging
                enum_desc = upload_pb2._TRACKSAMPLERESPONSE.enum_types[0]
                res_name = enum_desc.values_by_number[sample_res.response_code].name

                err_msg = "TrackSampleResponse code %s: %s" % (sample_res.response_code, res_name)

                if res_name == 'ALREADY_EXISTS':
                    # include the sid, too
                    # this shouldn't be relied on externally, but I use it in
                    # tests - being surrounded by parens is how it's matched
                    err_msg += "(%s)" % sample_res.server_track_id

                self.logger.warning("upload of '%s' rejected: %s", path, err_msg)
                not_uploaded[path] = err_msg

        #Send upload requests.
        if to_upload:
            #TODO reordering requests could avoid wasting time waiting for reup sync
            self._make_call(musicmanager.UpdateUploadState, 'start', self.uploader_id)

            for server_id, (path, track, do_not_rematch) in to_upload.items():
                #It can take a few tries to get an session.
                should_retry = True
                attempts = 0

                while should_retry and attempts < 10:
                    session = self._make_call(musicmanager.GetUploadSession,
                                              self.uploader_id, len(uploaded),
                                              track, path, server_id, do_not_rematch)
                    attempts += 1

                    got_session, error_details = \
                        musicmanager.GetUploadSession.process_session(session)

                    if got_session:
                        self.logger.info("got an upload session for '%s'", path)
                        break

                    should_retry, reason, error_code = error_details
                    self.logger.debug("problem getting upload session: %s\ncode=%s retrying=%s",
                                      reason, error_code, should_retry)

                    if error_code == 200 and do_not_rematch:
                        #reupload requests need to wait on a server sync
                        #200 == already uploaded, so force a retry in this case
                        should_retry = True

                    time.sleep(6)  # wait before retrying
                else:
                    err_msg = "GetUploadSession error %s: %s" % (error_code, reason)

                    self.logger.warning("giving up on upload session for '%s': %s", path, err_msg)
                    not_uploaded[path] = err_msg

                    continue  # to next upload

                #got a session, do the upload
                #this terribly inconsistent naming isn't my fault: Google--
                session = session['sessionStatus']
                external = session['externalFieldTransfers'][0]

                session_url = external['putInfo']['url']
                content_type = external.get('content_type', 'audio/mpeg')

                if track.original_content_type != locker_pb2.Track.MP3:
                    try:
                        self.logger.info("transcoding '%s' to mp3", path)
                        contents = utils.transcode_to_mp3(path, quality=transcode_quality)
                    except (IOError, ValueError) as e:
                        self.logger.warning("error transcoding %s: %s", path, e)
                        not_uploaded[path] = "transcoding error: %s" % e
                        continue
                else:
                    with open(path, 'rb') as f:
                        contents = f.read()

                upload_response = self._make_call(musicmanager.UploadFile,
                                                  session_url, content_type, contents)

                success = upload_response.get('sessionStatus', {}).get('state')
                if success:
                    uploaded[path] = server_id
                else:
                    #404 == already uploaded? serverside check on clientid?
                    self.logger.debug("could not finalize upload of '%s'. response: %s",
                                      path, upload_response)
                    not_uploaded[path] = 'could not finalize upload; details in log'

            self._make_call(musicmanager.UpdateUploadState, 'stopped', self.uploader_id)

        return uploaded, matched, not_uploaded


class Webclient(_Base):
    """Allows library management and streaming by posing as the
    music.google.com webclient.

    Uploading is not supported by this client (use the :class:`Musicmanager`
    to upload).
    """

    def __init__(self, debug_logging=True):
        self.session = gmusicapi.session.Webclient()

        super(Webclient, self).__init__(self.__class__.__name__, debug_logging)
        self.logout()

    def login(self, email, password):
        """Authenticates the webclient.
        Returns ``True`` on success, ``False`` on failure.

        :param email: eg ``'test@gmail.com'`` or just ``'test'``.
        :param password: password or app-specific password for 2-factor users.
          This is not stored locally, and is sent securely over SSL.

        Users of two-factor authentication will need to set an application-specific password
        to log in.
        """

        if not self.session.login(email, password):
            self.logger.info("failed to authenticate")
            return False

        self.logger.info("authenticated")

        return True

    def logout(self):
        return super(Webclient, self).logout()

    def change_playlist_name(self, playlist_id, new_name):
        """Changes the name of a playlist. Returns the changed id.

        :param playlist_id: id of the playlist to rename.
        :param new_title: desired title.
        """

        self._make_call(webclient.ChangePlaylistName, playlist_id, new_name)

        return playlist_id  # the call actually doesn't return anything.

    @utils.accept_singleton(dict)
    @utils.empty_arg_shortcircuit()
    def change_song_metadata(self, songs):
        """Changes the metadata for some :ref:`song dictionaries <songdict-format>`.
        Returns a list of the song ids changed.

        :param songs: a list of :ref:`song dictionaries <songdict-format>`,
          or a single :ref:`song dictionary <songdict-format>`.

        Generally, stick to these metadata keys:

        * ``rating``: set to 0 (no thumb), 1 (down thumb), or 5 (up thumb)
        * ``name``: use this instead of ``title``
        * ``album``
        * ``albumArtist``
        * ``artist``
        * ``composer``
        * ``disc``
        * ``genre``
        * ``playCount``
        * ``totalDiscs``
        * ``totalTracks``
        * ``track``
        * ``year``
        """

        res = self._make_call(webclient.ChangeSongMetadata, songs)

        return [s['id'] for s in res['songs']]

    def create_playlist(self, name):
        """Creates a new playlist. Returns the new playlist id.

        :param title: the title of the playlist to create.
        """

        return self._make_call(webclient.AddPlaylist, name)['id']

    def delete_playlist(self, playlist_id):
        """Deletes a playlist. Returns the deleted id.

        :param playlist_id: id of the playlist to delete.
        """

        res = self._make_call(webclient.DeletePlaylist, playlist_id)

        return res['deleteId']

    @utils.accept_singleton(basestring)
    @utils.empty_arg_shortcircuit()
    def delete_songs(self, song_ids):
        """Deletes songs from the entire library. Returns a list of deleted song ids.

        :param song_ids: a list of song ids, or a single song id.
        """

        res = self._make_call(webclient.DeleteSongs, song_ids)

        return res['deleteIds']

    def get_all_songs(self, incremental=False):
        """Returns a list of :ref:`song dictionaries <songdict-format>`.

        :param incremental: if True, return a generator that yields lists
          of at most 2500 :ref:`song dictionaries <songdict-format>`
          as they are retrieved from the server. This can be useful for
          presenting a loading bar to a user.
        """

        to_return = self._get_all_songs()

        if not incremental:
            to_return = [song for chunk in to_return for song in chunk]

        return to_return

    def _get_all_songs(self):
        """Return a generator of song chunks."""

        get_next_chunk = True
        lib_chunk = {'continuationToken': None}

        while get_next_chunk:
            lib_chunk = self._make_call(webclient.GetLibrarySongs,
                                        lib_chunk['continuationToken'])

            yield lib_chunk['playlist']  # list of songs of the chunk

            get_next_chunk = 'continuationToken' in lib_chunk

    def get_playlist_songs(self, playlist_id):
        """Returns a list of :ref:`song dictionaries <songdict-format>`,
        which include ``playlistEntryId`` keys for the given playlist.

        :param playlist_id: id of the playlist to load.

        This will return ``[]`` if the playlist id does not exist.
        """

        res = self._make_call(webclient.GetPlaylistSongs, playlist_id)
        return res['playlist']

    def get_all_playlist_ids(self, auto=True, user=True):
        """Returns a dictionary that maps playlist types to dictionaries.

        :param auto: create an ``'auto'`` subdictionary entry.
          Currently, this will just map to ``{}``; support for 'Shared with me' and
          'Google Play recommends' is on the way (
          `#102 <https://github.com/simon-weber/Unofficial-Google-Music-API/issues/102>`__).

          Other auto playlists are not stored on the server, but calculated by the client.
          See `this gist <https://gist.github.com/simon-weber/5007769>`__ for sample code for
          'Thumbs Up', 'Last Added', and 'Free and Purchased'.

        :param user: create a user ``'user'`` subdictionary entry for user-created playlists.
          This includes anything that appears on the left side 'Playlists' bar (notably, saved
          instant mixes).

        User playlist names will be unicode strings.

        Google Music allows multiple user playlists with the same name, so the ``'user'`` dictionary
        will map onto lists of ids. Here's an example response::

            {
                'auto':{},

                'user':{
                    u'Some Song Mix':[
                        u'14814747-efbf-4500-93a1-53291e7a5919'
                    ],

                    u'Two playlists have this name':[
                        u'c89078a6-0c35-4f53-88fe-21afdc51a414',
                        u'86c69009-ea5b-4474-bd2e-c0fe34ff5484'
                    ]
                }
            }

        There is currently no support for retrieving automatically-created instant mixes
        (see issue `#67 <https://github.com/simon-weber/Unofficial-Google-Music-API/issues/67>`__).

        """

        playlists = {}

        if auto:
            #playlists['auto'] = self._get_auto_playlists()
            playlists['auto'] = {}
        if user:
            res = self._make_call(webclient.GetPlaylistSongs, 'all')
            playlists['user'] = self._playlist_list_to_dict(res['playlists'])

        return playlists

    def _playlist_list_to_dict(self, pl_list):
        ret = {}

        for name, pid in ((p["title"], p["playlistId"]) for p in pl_list):
            if not name in ret:
                ret[name] = []
            ret[name].append(pid)

        return ret

    def _get_auto_playlists(self):
        """For auto playlists, returns a dictionary which maps autoplaylist name to id."""

        #Auto playlist ids are hardcoded in the wc javascript.
        #When testing, an incorrect name here will be caught.
        return {u'Thumbs up': u'auto-playlist-thumbs-up',
                u'Last added': u'auto-playlist-recent',
                u'Free and purchased': u'auto-playlist-promo'}

    def get_song_download_info(self, song_id):
        """Returns a tuple: ``('<url>', <download count>)``.

        :param song_id: a single song id.

        ``url`` will be ``None`` if the download limit is exceeded.

        GM allows 2 downloads per song. The download count may not always be accurate,
        and the 2 download limit seems to be loosely enforced.

        This call alone does not count towards a download -
        the count is incremented when ``url`` is retrieved.
        """

        #TODO the protocol expects a list of songs - could extend with accept_singleton
        info = self._make_call(webclient.GetDownloadInfo, [song_id])
        url = info.get('url')

        return (url, info["downloadCounts"][song_id])

    def get_stream_url(self, song_id):
        """Returns a url that points to a streamable version of this song.

        :param song_id: a single song id.

        While acquiring the url requires authentication, retreiving the
        url contents does not.

        However, there are limitation as to how the stream url can be used:
            * the url expires after about a minute
            * only one IP can be streaming music at once.
              Other attempts will get an http 403 with
              ``X-Rejected-Reason: ANOTHER_STREAM_BEING_PLAYED``.

        *This is only intended for streaming*. The streamed audio does not contain metadata.
        Use :func:`get_song_download_info` to download complete files with metadata.
        """

        res = self._make_call(webclient.GetStreamUrl, song_id)

        return res['url']

    def copy_playlist(self, playlist_id, copy_name):
        """Copies the contents of a playlist to a new playlist. Returns the id of the new playlist.

        :param playlist_id: id of the playlist to be copied.
        :param copy_name: the name of the new copied playlist.

        This is useful for making backups of playlists before modifications.
        """

        orig_tracks = self.get_playlist_songs(playlist_id)

        new_id = self.create_playlist(copy_name)
        self.add_songs_to_playlist(new_id, [t["id"] for t in orig_tracks])

        return new_id

    def change_playlist(self, playlist_id, desired_playlist, safe=True):
        """Changes the order and contents of an existing playlist.
        Returns the id of the playlist when finished -
        this may be the same as the argument in the case of a failure and recovery.

        :param playlist_id: the id of the playlist being modified.
        :param desired_playlist: the desired contents and order as a list of
          :ref:`song dictionaries <songdict-format>`, like is returned
          from :func:`get_playlist_songs`.

        :param safe: if ``True``, ensure playlists will not be lost if a problem occurs.
          This may slow down updates.

        The server only provides 3 basic playlist mutations: addition, deletion, and reordering.
        This function will use these to automagically apply the desired changes.

        However, this might involve multiple calls to the server, and if a call fails,
        the playlist will be left in an inconsistent state.
        The ``safe`` option makes a backup of the playlist before doing anything, so it can be
        rolled back if a problem occurs. This is enabled by default.
        This might slow down updates of very large playlists.

        There will always be a warning logged if a problem occurs, even if ``safe`` is ``False``.
        """

        #We'll be modifying the entries in the playlist, and need to copy it.
        #Copying ensures two things:
        # 1. the user won't see our changes
        # 2. changing a key for one entry won't change it for another - which would be the case
        #     if the user appended the same song twice, for example.
        desired_playlist = [copy.deepcopy(t) for t in desired_playlist]
        server_tracks = self.get_playlist_songs(playlist_id)

        if safe:
            #Make a backup.
            #The backup is stored on the server as a new playlist with "_gmusicapi_backup"
            # appended to the backed up name.
            names_to_ids = self.get_all_playlist_ids()['user']
            playlist_name = (ni_pair[0]
                             for ni_pair in names_to_ids.iteritems()
                             if playlist_id in ni_pair[1]).next()

            backup_id = self.copy_playlist(playlist_id, playlist_name + u"_gmusicapi_backup")

        try:
            #Counter, Counter, and set of id pairs to delete, add, and keep.
            to_del, to_add, to_keep = \
                tools.find_playlist_changes(server_tracks, desired_playlist)

            ##Delete unwanted entries.
            to_del_eids = [pair[1] for pair in to_del.elements()]
            if to_del_eids:
                self._remove_entries_from_playlist(playlist_id, to_del_eids)

            ##Add new entries.
            to_add_sids = [pair[0] for pair in to_add.elements()]
            if to_add_sids:
                new_pairs = self.add_songs_to_playlist(playlist_id, to_add_sids)

                ##Update desired tracks with added tracks server-given eids.
                #Map new sid -> [eids]
                new_sid_to_eids = {}
                for sid, eid in new_pairs:
                    if not sid in new_sid_to_eids:
                        new_sid_to_eids[sid] = []
                    new_sid_to_eids[sid].append(eid)

                for d_t in desired_playlist:
                    if d_t["id"] in new_sid_to_eids:
                        #Found a matching sid.
                        match = d_t
                        sid = match["id"]
                        eid = match.get("playlistEntryId")
                        pair = (sid, eid)

                        if pair in to_keep:
                            to_keep.remove(pair)  # only keep one of the to_keep eids.
                        else:
                            match["playlistEntryId"] = new_sid_to_eids[sid].pop()
                            if len(new_sid_to_eids[sid]) == 0:
                                del new_sid_to_eids[sid]

            ##Now, the right eids are in the playlist.
            ##Set the order of the tracks:

            #The web client has no way to dictate the order without block insertion,
            # but the api actually supports setting the order to a given list.
            #For whatever reason, though, it needs to be set backwards; might be
            # able to get around this by messing with afterEntry and beforeEntry parameters.
            if desired_playlist:
                #can't *-unpack an empty list
                sids, eids = zip(*tools.get_id_pairs(desired_playlist[::-1]))

                if sids:
                    self._make_call(webclient.ChangePlaylistOrder, playlist_id, sids, eids)

            ##Clean up the backup.
            if safe:
                self.delete_playlist(backup_id)

        except CallFailure:
            self.logger.info("a subcall of change_playlist failed - "
                             "playlist %s is in an inconsistent state", playlist_id)

            if not safe:
                raise  # there's nothing we can do
            else:  # try to revert to the backup
                self.logger.info("attempting to revert changes from playlist "
                                 "'%s_gmusicapi_backup'", playlist_name)

                try:
                    self.delete_playlist(playlist_id)
                    self.change_playlist_name(backup_id, playlist_name)
                except CallFailure:
                    self.logger.warning("failed to revert failed change_playlist call on '%s'",
                                        playlist_name)
                    raise
                else:
                    self.logger.info("reverted changes safely; playlist id of '%s' is now '%s'",
                                     playlist_name, backup_id)
                    playlist_id = backup_id

        return playlist_id

    @utils.accept_singleton(basestring, 2)
    @utils.empty_arg_shortcircuit(position=2)
    def add_songs_to_playlist(self, playlist_id, song_ids):
        """Appends songs to a playlist.
        Returns a list of (song id, playlistEntryId) tuples that were added.

        :param playlist_id: id of the playlist to add to.
        :param song_ids: a list of song ids, or a single song id.
        """

        res = self._make_call(webclient.AddToPlaylist, playlist_id, song_ids)
        new_entries = res['songIds']

        return [(e['songId'], e['playlistEntryId']) for e in new_entries]

    @utils.accept_singleton(basestring, 2)
    @utils.empty_arg_shortcircuit(position=2)
    def remove_songs_from_playlist(self, playlist_id, sids_to_match):
        """Removes all copies of the given song ids from a playlist.
        Returns a list of removed (sid, eid) pairs.

        :param playlist_id: id of the playlist to remove songs from.
        :param sids_to_match: a list of song ids to match, or a single song id.

        This does *not always* the inverse of a call to :func:`add_songs_to_playlist`,
        since multiple copies of the same song are removed. For more control in this case,
        get the playlist tracks with :func:`get_playlist_songs`, modify the list of tracks,
        then use :func:`change_playlist` to push changes to the server.
        """

        playlist_tracks = self.get_playlist_songs(playlist_id)
        sid_set = set(sids_to_match)

        matching_eids = [t["playlistEntryId"]
                         for t in playlist_tracks
                         if t["id"] in sid_set]

        if matching_eids:
            #Call returns "sid_eid" strings.
            sid_eids = self._remove_entries_from_playlist(playlist_id,
                                                          matching_eids)
            return [s.split("_") for s in sid_eids]
        else:
            return []

    @utils.accept_singleton(basestring, 2)
    @utils.empty_arg_shortcircuit(position=2)
    def _remove_entries_from_playlist(self, playlist_id, entry_ids_to_remove):
        """Removes entries from a playlist. Returns a list of removed "sid_eid" strings.

        :param playlist_id: the playlist to be modified.
        :param entry_ids: a list of entry ids, or a single entry id.
        """

        #GM requires the song ids in the call as well; find them.
        playlist_tracks = self.get_playlist_songs(playlist_id)
        remove_eid_set = set(entry_ids_to_remove)

        e_s_id_pairs = [(t["id"], t["playlistEntryId"])
                        for t in playlist_tracks
                        if t["playlistEntryId"] in remove_eid_set]

        num_not_found = len(entry_ids_to_remove) - len(e_s_id_pairs)
        if num_not_found > 0:
            self.logger.warning("when removing, %d entry ids could not be found in playlist id %s",
                                num_not_found, playlist_id)

        #Unzip the pairs.
        sids, eids = zip(*e_s_id_pairs)

        res = self._make_call(webclient.DeleteSongs, sids, playlist_id, eids)

        return res['deleteIds']

    def search(self, query):
        """Queries the server for songs and albums.

        **WARNING**: Google no longer uses this endpoint in their client;
        it may stop working or be removed from gmusicapi without warning.
        In addition, it is known to occasionally return unexpected results.
        See `#114
        <https://github.com/simon-weber/Unofficial-Google-Music-API/issues/114>`__
        for more information.

        Instead of using this call, retrieve all tracks with :func:`get_all_songs`
        and search them locally.  `This gist
        <https://gist.github.com/simon-weber/5007769>`__ has some examples of
        simple linear-time searches.

        :param query: a string keyword to search with. Capitalization and punctuation are ignored.

        The results are returned in a dictionary, arranged by how they were found.
        ``artist_hits`` and ``song_hits`` return a list of
        :ref:`song dictionaries <songdict-format>`, while ``album_hits`` entries
        have a different structure.

        For example, a search on ``'cat'`` could return::

            {
                "album_hits": [
                    {
                        "albumArtist": "The Cat Empire",
                        "albumName": "Cities: The Cat Empire Project",
                        "artistName": "The Cat Empire",
                        "imageUrl": "//ssl.gstatic.com/music/fe/[...].png"
                        # no more entries
                    },
                ],
                "artist_hits": [
                    {
                        "album": "Cinema",
                        "artist": "The Cat Empire",
                        "id": "c9214fc1-91fa-3bd2-b25d-693727a5f978",
                        "title": "Waiting"
                        # ... normal song dictionary
                    },
                ],
                "song_hits": [
                    {
                        "album": "Mandala",
                        "artist": "RX Bandits",
                        "id": "a7781438-8ec3-37ab-9c67-0ddb4115f60a",
                        "title": "Breakfast Cat",
                        # ... normal song dictionary
                    },
                ]
            }

        """

        res = self._make_call(webclient.Search, query)['results']

        return {"album_hits": res["albums"],
                "artist_hits": res["artists"],
                "song_hits": res["songs"]}

    @utils.accept_singleton(basestring)
    @utils.empty_arg_shortcircuit()
    def report_incorrect_match(self, song_ids):
        """Equivalent to the 'Fix Incorrect Match' button, this requests re-uploading of songs.
        Returns the song_ids provided.

        :param song_ids: a list of song ids to report, or a single song id.

        Note that if you uploaded a song through gmusicapi, it won't be reuploaded
        automatically - this currently only works for songs uploaded with the Music Manager.
        See issue `#89 <https://github.com/simon-weber/Unofficial-Google-Music-API/issues/89>`__.

        This should only be used on matched tracks (``song['type'] == 6``).
        """

        self._make_call(webclient.ReportBadSongMatch, song_ids)

        return song_ids

    @utils.accept_singleton(basestring)
    @utils.empty_arg_shortcircuit()
    def upload_album_art(self, song_ids, image_filepath):
        """Uploads an image and sets it as the album art for songs.

        :param song_ids: a list of song ids, or a single song id.
        :param image_filepath: filepath of the art to use. jpg and png are known to work.

        This function will *always* upload the provided image, even if it's already uploaded.
        If the art is already uploaded and set for another song, copy over the
        value of the ``'albumArtUrl'`` key using :func:`change_song_metadata` instead.
        """

        res = self._make_call(webclient.UploadImage, image_filepath)
        url = res['imageUrl']

        song_dicts = [dict((('id', id), ('albumArtUrl', url))) for id in song_ids]

        return self.change_song_metadata(song_dicts)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Utilities used in testing."""

from glob import glob
import logging
import numbers
import os
import re


#from gmusicapi.api import Api
from gmusicapi.protocol.metadata import md_expectations
from gmusicapi.utils import utils

log = utils.DynamicClientLogger(__name__)


#A regex for the gm id format, eg:
#c293dd5a-9aa9-33c4-8b09-0c865b56ce46
hex_set = "[0-9a-f]"
gm_id_regex = re.compile(("{h}{{8}}-" +
                         ("{h}{{4}}-" * 3) +
                         "{h}{{12}}").format(h=hex_set))

#Get the absolute paths of the test files, which are located in the same
# directory as this file.
cwd = os.getcwd()
test_file_dir = os.path.dirname(os.path.abspath(__file__))
os.chdir(test_file_dir)

_audio_filenames = glob(u'audiotest*')
mp3_filenames = [os.path.abspath(fn) for fn in _audio_filenames if fn.endswith('.mp3')]
small_mp3 = os.path.abspath(u'audiotest_small.mp3')
image_filename = os.path.abspath(u'imagetest_10x10_check.png')

os.chdir(cwd)


class NoticeLogging(logging.Handler):
    """A log handler that, if asked to emit, will set
    ``self.seen_message`` to True.
    """

    def __init__(self):
        logging.Handler.__init__(self)  # cannot use super in py 2.6; logging is still old-style
        self.seen_message = False

    def emit(self, record):
        self.seen_message = True


def new_test_client(cls, **kwargs):
    """Make an instance of a client, login, and return it.

    kwargs are passed through to cls.login().
    """

    client = cls(debug_logging=True)
    client.login(**kwargs)

    return client


def modify_md(md_name, val):
    """Returns a value of the same type as val that will not equal val."""

    #Check for metadata that must get specific values.
    if md_expectations[md_name].allowed_values is not None:
        #Assume old_val is a possible value, and return
        # the value one modulus index after it.

        possible = md_expectations[md_name].allowed_values
        val_i = 0
        try:
            val_i = possible.index(val)
        except ValueError:
            log.warning("non-allowed metadata value '%s' for key %s", val, md_name)

        return possible[(val_i + 1) % len(possible)]

    #Generic handlers for other data types.
    if isinstance(val, basestring):
        return val + "_mod"

    #Need to check for bool first, bools are instances of Number for some reason.
    elif isinstance(val, bool):
        return not val
    elif isinstance(val, numbers.Number):
        return val + 1
    else:
        raise TypeError("modify expects only strings, numbers, and bools")


def md_entry_same(entry_name, s1, s2):
    """Returns (s1 and s2 have the same value for entry_name?, message)."""

    s1_val = s1[entry_name]
    s2_val = s2[entry_name]

    return (s1_val == s2_val, "(" + entry_name + ") " + repr(s1_val) + ", " + repr(s2_val))


def is_gm_id(s):
    """Returns True if the given string is in Google Music id form."""
    return re.match(gm_id_regex, s) is not None


def is_song(d):
    """Returns True is the given dict is a GM song dict."""
    #Not really precise, but should be good enough.
    return is_gm_id(d["id"])


def is_song_list(lst):
    return all(map(is_song, lst))


def is_id_list(lst):
    """Returns True if the given list is made up of all strings in GM id form."""
    return all(map(is_gm_id, lst))


def is_id_pair_list(lst):
    """Returns True if the given list is made up of all (id, id) pairs."""
    a, b = zip(*lst)
    return is_id_list(a+b)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
All known information on metadata is exposed in ``gmusicapi.protocol.metadata.md_expectations``.

This holds a mapping of *name* to *Expectation*, where *Expectation* has
the following fields:

*name*
  key name in the song dictionary (equal to the *name* keying ``md_expectations``).

*type*:
    a string holding a `validictory <https://github.com/sunlightlabs/validictory>`__ type.

    Possible values:
      :'string':
          str and unicode objects
      :'integer':
          ints, longs
      :'number':
          ints, longs and floats
      :'boolean':
          bools
      :'object':
          dicts
      :'array':
          lists and tuples
      :'null':
          ``None``
      :'any':
          any type is possible

*mutable*:
  ``True`` if client can change the value.

*optional*:
  ``True`` if the key is not guaranteed to be present.

*volatile*:
  ``True`` if the key's value can change between observations without client mutation.

*depends_on*:
  the name of the key we transform to take our value from, or ``None``.

  These fields can never be changed: they are automatically set to
  a modified form of some other field's value.
  See *dependent_transformation* for more information.

*dependent_transformation*:
  ``None``, or a function ``lambda dependent_value: our_value``.

  For example, the ``artistNorm`` field is automatically set to the lowercase
  of the ``artist`` field.
  So, ``artistNorm.depends_on == 'artist'``, and the *dependent_transformation* for
  ``artistNorm`` can be written as ``lambda artist: artist.lower()``.

*allowed_values*:
  sequence of allowed values.

*explanation*:
  an explanatory string, typically empty for obvious fields.

The above information is used to generate the documentation below.
If you find an example to clarify these expectations, please `submit an issue
<https://github.com/simon-weber/Unofficial-Google-Music-API/issues>`__.
"""

from collections import defaultdict, namedtuple


_Expectation = namedtuple(
    '_Expectation',
    [
        'name', 'type', 'mutable', 'optional', 'volatile',
        'depends_on', 'dependent_transformation',
        'allowed_values', 'explanation'
    ]
)


class Expectation(_Expectation):
    """Instantiated to represent information about a single metadata key."""
    #This class just wraps the namedtuple to provide easy construction and some methods.

    def __new__(cls, name, type, mutable, optional, volatile=False,
                depends_on=None, dependent_transformation=None,
                allowed_values=None, explanation=''):
        return cls.__bases__[0].__new__(
            cls,
            name, type, mutable, optional, volatile,
            depends_on, dependent_transformation,
            allowed_values, explanation
        )

    def get_schema(self):
        """Return a validictory schema for this key."""
        schema = {}
        schema["type"] = self.type
        if self.type == "string":
            schema["blank"] = True  # allow blank strings
        if self.optional:
            schema["required"] = False

        return schema

#: All the expectations.
_all_expts = [
    Expectation(name, 'string', mutable=True, optional=False) for name in
    (
        'composer', 'album', 'albumArtist', 'genre', 'name', 'artist', 'comment',
    )
] + [
    Expectation(name, 'integer', mutable=True, optional=True) for name in
    (
        'disc', 'year', 'track', 'totalTracks', 'totalDiscs', 'explicitType',
    )
] + [
    Expectation(name, type_str, mutable=False, optional=False, explanation=explain)
    for (name, type_str, explain) in
    (
        ('durationMillis', 'integer',
         'length of a song in milliseconds.'),

        ('id', 'string',
         'a per-user unique id for this song; sometimes referred to as *server id* or *song id*.'),

        ('deleted', 'boolean', ''),
        ('creationDate', 'integer', ''),
        ('type', 'integer',
         'An enum: 1: free/purchased, 2: uploaded/not matched, 6: uploaded/matched'),

        ('beatsPerMinute', 'integer',
         "the server does not calculate this - it's just what was in track metadata"),

        ('subjectToCuration', 'boolean', 'meaning unknown.'),
        ('matchedId', 'string', 'meaning unknown; related to scan and match?'),
        ('recentTimestamp', 'integer', 'meaning unknown.'),
    )
] + [
    Expectation(name, type_str, mutable=False, optional=True, explanation=explain)
    for (name, type_str, explain) in
    (
        ('storeId', 'string', 'an id of a matching track in the Play Store.'),
        ('reuploading', 'boolean', 'scan-and-match reupload in progress.'),
        ('albumMatchedId', 'string', 'id of matching album in the Play Store?'),
        ('pending', 'boolean', 'unsure; server processing (eg for store match) pending?'),
        ('url', 'string',  'meaning unknown.'),
        ('bitrate', 'integer', "bitrate in kilobytes/second (eg 320)."),
        ('playlistEntryId', 'string', 'identifies position in the context of a playlist.'),
        ('albumArtUrl', 'string', "if present, the url of an image for this song's album art."),
        ('artistMatchedId', 'string', 'id of a matching artist in the Play Store?'),
        ('albumPlaybackTimestamp', 'integer', 'the last time this album was played?'),
        ('origin', 'array', '???'),
        ('artistImageBaseUrl', 'string', 'like albumArtUrl, but for the artist. May be blank.'),
    )
] + [
    Expectation(name + 'Norm', 'string', mutable=False, optional=False,
                depends_on=name,
                dependent_transformation=lambda x: x.lower(),
                explanation="automatically set to lowercase of *%s*." % name)
    for name in
    (
        'artist', 'albumArtist', 'album'
    )
] + [
    # 0, 1, 5: no, down, up thumbs
    Expectation('rating', 'integer', mutable=True,
                optional=False, allowed_values=tuple(range(6)),
                explanation='0 == no thumb, 1 == down thumb, 5 == up thumb.'),

    Expectation('lastPlayed', 'integer', mutable=False, optional=True, volatile=True),
    Expectation('playCount', 'integer', mutable=True, optional=False),

    Expectation('title', 'string', mutable=False, optional=False,
                depends_on='name', dependent_transformation=lambda x: x,
                explanation='misleading! automatically set to *name*.'),

    Expectation('titleNorm', 'string', mutable=False, optional=False,
                depends_on='name', dependent_transformation=lambda x: x.lower(),
                explanation='misleading! automatically set to lowercase of *name*.'),
]

#Create the dict for client code. If they look up something we don't know about,
# give them a flexible immutable key.
_immutable_key = lambda: Expectation('unknown', 'any', mutable=False, optional=True)
md_expectations = defaultdict(_immutable_key)
for expt in _all_expts:
    md_expectations[expt.name] = expt


#This code is a super-hack. KnownMetadataFields exists _purely_ for documentation.

#We want dynamic documentation based on _all_expts, but __doc__ isn't a writable field
#for non-{function, class, module} objects. So, we create a dummy class and dynamically
#create its docstring to be arbitrary reST that documents our expectations.

def detail_line(e):
    """Given an expectation, return a readable one-line explanation of it."""
    fields = [fname for fname in ('mutable', 'optional', 'volatile')
              if getattr(e, fname, None)]

    if e.depends_on:
        fields.append("depends_on=%s" % e.depends_on)

    line = ', '.join(fields)
    if line:
        line = "*(%s)*" % line

    return line

#Note the hackiness of this class.
dynamic_docs = """
**This class exists only for documentation; do not try to import it.**

Instead, client code should use ``gmusicapi.protocol.metadata.md_expectations``.

See `the code <https://github.com/simon-weber/Unofficial-Google-Music-API/blob
/develop/gmusicapi/protocol/metadata.py>`__ for an explanation of this hack.
Ideas to clean this up are welcomed.

"""

#Create a reST definition list dynamically.
dynamic_docs += '\n\n'.join(
    ("*{name}*\n"
     "  {type} {details}\n\n"
     "  {explanation}").format(
         name=e.name,
         type=e.type,
         details=detail_line(e),
         explanation=e.explanation,
     ) for e in sorted(_all_expts, key=lambda e: e.name)
)


KnownMetadataFields = type('KnownMetadataFields', (defaultdict,), {'__doc__': dynamic_docs})
